///File : DevisManager.hpp
//------------------------------------------------------------------------------
/*
    Copyright (C) 2009 Colin GILLE / congelli501

    This file is part of Devis Maker.
    Devis Maker is sold under Devis Maker license (read COPYING file)
*/
//------------------------------------------------------------------------------
#if not defined DEVIS_MANAGER_HPP
#define DEVIS_MANAGER_HPP
//------------------------------------------------------------------------------
#include <string>
#include <iostream>
#include <vector>
#include <map>
//------------------------------------------------------------------------------
#if not defined TIXML_USE_TICPP
    #define TIXML_USE_TICPP
#endif
#include "ticpp/ticpp.h"
//------------------------------------------------------------------------------
#include "ItemSectionDBBase.hpp"
#include "ConfigManager.hpp"
//------------------------------------------------------------------------------
#include "config.hpp"
#include "buildVersionOptions.hpp"
//------------------------------------------------------------------------------
#define BACKUP_NB 20

enum
{
    BACKUP_SECTION_MODIFIED,
    BACKUP_SECTION_ADDED,
    BACKUP_SECTION_REMOVED,
    BACKUP_SECTION_RENAMED,
    BACKUP_SECTION_MOVED,
    BACKUP_NEW
};

struct StructTVALine
{
    std::string t;
    double TVA;
    double base;
    double total;
};

struct StructItem
{
    std::string ref;
    double quantity;
};

struct StructSection
{
    std::string sectionID;
    std::vector<StructItem> items;
};

// Backup
struct StructBackup
{
    // Data
    std::vector<StructSection> sections;
    bool isDevisModified;

    // Type
    int type;
    std::string infoStr;
};

struct StructDevis
{
    ConfigManager cfg;
    std::string filename;
    std::vector<StructSection> sections;

    bool isDevisModified;

    // Backup
    std::vector<StructBackup> backup;
    int backupPos; // La position actuelle dans le tableau...
};

class DevisManager : public ItemSectionDBBase
{
public:
    // Init
    DevisManager(const std::string &sectionDBPath, const std::string &itemDBPath, const std::string &cfgPath );
    void ReloadSectionItemDB(const std::string &sectionDBPath, const std::string &itemDBPath);

    // Clean devs
    bool CleanDevis();

    // Gestion fichier
    void Save(const std::string &filename = "");
    void LoadFromFile(const std::string &filename);
    void New();
    std::string GetFilename() const;
    void Close();

    // Flags
    bool IsOpen() const;
    bool IsModified() const;

    // Infos Generales & Devis
    ConfigManager *GetDevisCfg();
    GeneralCfg *GetGeneralCfg();

    // Sections (Gestion stock)
    bool NewSection(const std::string &sectionID);
    int GetSectionCount() const;
    int GetItemCountInAllSections() const;
    std::vector<std::string> GetSectionsList() const;
    int GetSectionItemsNumber(const size_t &posSection) const;
    std::string GetDevisSectionName(const size_t &posSection) const;
    bool RemoveSection(const size_t &posSection);
    bool RenameSection(const size_t &posSection, const std::string &newName);
    bool SwapSections(const size_t &posSection1, const size_t &posSection2);
    double GetSectionPriceHT(const size_t &posSection) const;
    std::string GetSectionName(const size_t &posSection) const;

    // Items (Gestion stock)
    std::vector<std::string> GetItemsList(const size_t &posSection) const;
    bool NewItem(const int &posSection);
    double GetItemQuantity(const int &posSection, const size_t &posItem) const;
    bool RemoveItem(const size_t &posSection, const size_t &posItem);
    void UpdateItem(const size_t &posSection, const size_t &posItem, const std::string &itemID, const double &quantity);
    std::string GetItemRef(const size_t &posSection, const size_t &posItem) const;
    double GetItemPriceWithQuantityHT(const size_t &posSection, const size_t &posItem) const;
    double GetItemPriceWithQuantityEcoPartHT(const size_t &posSection, const size_t &posItem) const;

    // Prices
    double GetSectionsPriceHT() const;
    double GetDumpChargeHT() const;
    double GetPortHT() const;
    double GetTotalBrutHT() const;
    double GetPriceDiscountRateHT() const;
    double GetDiscountHT() const;
    double GetTotalNetHT() const;
    double GetTVAPrice() const;
    double GetTotalTTC() const;
    void AddToTVATbl(std::vector<StructTVALine> &tempTbl, const double &currentBase, const double &currentTVA) const;

    double GetPaymentOn() const;
    double GetTotalToPay() const;
    double GetEcoPartTotalPrice() const;

    // Mise en forme
    std::vector<StructTVALine> GetTVATbl() const;
    std::string GetItemTVAid(const int &posSection, const size_t &posItem, const std::vector<StructTVALine> &TVATbl) const;


    // Undo / Redo
    bool BackupCanGoTo(const int &pos) const;
    void BackupSaveCurrent(const int &type, const std::string &infoStr = "");
    void BackupLoad(const int &pos);
    int BackupGetType(const int &pos) const;
    std::string BackupGetInfoStr(const int &pos) const;
    void BackupClear();
    void BackupStart(const int &type, const std::string &infoStr = "");

    // Gestion de plusieurs devis
    #if defined USE_MULTIPLE_CONFIG
    int GetLastCreatedDevisId() const;

    int GetCurrentDevisId() const;
    void SetCurrentDevisId(const int &id);
    #endif

protected:
    // Sauvegarde / ticpp
    ticpp::Element *PrepareInfo(const std::string &nom, const std::string &info) const;
    ticpp::Element *PrepareSection(const int posSection) const;

    // Throw
    void ThrowIfNotOpenAndOk();

    // Gestion de plusieurs devis
    #if defined USE_MULTIPLE_CONFIG
    void RemoveDevis(const int &id);
    int CopyDevis(const int &id);
    int NewDevis();
    #endif

    #if not defined USE_MULTIPLE_CONFIG
    // Flags
    bool m_isOpen;
    #endif

    // Devis
    #if defined USE_MULTIPLE_CONFIG
    std::map<int, StructDevis> m_dvsMap;
    int m_currentDevisId;
    int m_LastDevisId;
    #else
    StructDevis m_realDvs;
    #endif
    StructDevis *m_dvs;

    // Global Cfg
    GeneralCfg *m_cfg;
    GeneralCfg m_realCfg;
};

#endif //DEVIS_MANAGER_HPP
